#include "std.h"
#include "bg.h"
#include "spr_arm.h"
#include "key.h"
#include "pcg.h"
#include "intr_arm.h"
#include "task.h"
#include "task_sys.h"
#include "task_myship.h"
#include "dmachain_arm.h"
#include "mymem.h"
#include "res.h"
#include "main.h"


//---------------------------------------------------------------------------
ST_STATUS Status;
u16 State;

//---------------------------------------------------------------------------
EWRAM_CODE int main()
{
	State = STATE_INIT;

	for(;;)
	{
		switch(State)
		{
			case STATE_INIT:  StateInit();  break;
			case STATE_RESET: StateReset(); break;
			case STATE_EXEC:  StateExec();  break;

			default:
				SystemError("[Err] main State", NULL);
				break;
		}
	}

	return 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateInit()
{
	REG_WSCNT = 0x4317;

	State = STATE_RESET;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateReset()
{
	BgInit();
	SprInit();
	KeyInit();
	PcgInit();
	TaskInit();
	MyMemInit();
	StatusInit();
	DmaChainInit();

	SprSetAnime(sprAnimeTable);
	PcgSetTileData(&spr_chr);

	TaskSysAlloc();
	TaskMyShipAlloc();

	IntrInit();
	IntrSetHandler(IRQ_VBLANK, SprVblank);
	IntrSetHandler(IRQ_VCOUNT, SprVcount);
	IntrStart();

	State = STATE_EXEC;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateExec()
{
	ST_STATUS* p = &Status;

	for(;;)
	{
		SystemCall(5);

		p->fpsCount++;
		p->gameTime++;
		p->gameFrame++;

		if(p->gameFrame < p->frameCount)
		{
			p->delay     = p->frameCount - p->gameFrame;
			p->gameFrame = p->frameCount;
		}
		else
		{
			p->delay = 0;
		}

		KeyChk();
		SprClean();
		PcgClean();

		TaskSort();
		TaskRun();
		TaskDraw();

		DmaChainTransfer();
//		TRACEOUT("%d\n", REG_VCOUNT);

	}

	State = STATE_RESET;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StatusInit()
{
	_Memset(&Status,  0x00, sizeof(ST_STATUS));

	Status.bulletWait  =  6;
	Status.bulletSpeed =  8;
	Status.bulletChr   =  0;
	Status.deg         = 12;
}


//---------------------------------------------------------------------------
EWRAM_CODE void SystemError(char* msg1, char* msg2)
{

	for(;;){}
}
