#include "bg.h"
#include "std.h"
#include "pcg.h"
#include "main.h"
#include "dmachain_arm.h"
#include "spr_arm.h"


//---------------------------------------------------------------------------
//main.c
extern ST_STATUS Status;

ST_SPR_BUF     SprBuf[2] EWRAM_BSS;
ST_SPR_STATUS  SprStatus;
ST_SPR_ANIME** ppSprAnime;
u32            dummy;

const ST_SPRITE32 unVisible = {
	((OBJ_DISABLE | SCREEN_HEIGHT) << 16) | SCREEN_WIDTH, 0
};
//---------------------------------------------------------------------------
EWRAM_CODE void SprInit()
{
	_Memset(&SprBuf, 0x00, sizeof(ST_SPR_BUF) * 2);
	_Memset(&SprStatus, 0x00, sizeof(ST_SPR_STATUS));

	SprStatus.dIdx = 0;
	SprStatus.wIdx = 1;
	dummy = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void SprSetAnime(void* pp)
{
	ppSprAnime = pp;
}
//---------------------------------------------------------------------------
IWRAM_CODE void SprClean()
{
	REG_DMA3SAD = (u32)&dummy;
	REG_DMA3DAD = (u32)&SprBuf[SprStatus.dIdx];
	REG_DMA3CNT = (u32)(sizeof(ST_SPR_BUF) / 2) | (DMA_SRC_FIXED | DMA_DST_INC | DMA16 | DMA_ENABLE);

	REG_DMA3SAD = (u32)&dummy;
	REG_DMA3DAD = (u32)OAM;
	REG_DMA3CNT = (u32)4*MAX_SPR_RASTER_CNT | (DMA_SRC_FIXED | DMA_DST_INC | DMA16 | DMA_ENABLE);

	SprStatus.dIdx   = (SprStatus.dIdx + 1) & 0x1;
	SprStatus.wIdx   = (SprStatus.wIdx + 1) & 0x1;
	SprStatus.allCnt = 0;
}

//---------------------------------------------------------------------------
IWRAM_CODE void SprSetBuf(u16 idx, s32 x, s32 y, u16 pal)
{
	ST_SPR_PAT* pPat = &ppSprAnime[idx]->p[0];
	u16 num = ppSprAnime[idx]->regCnt;
	u16 i;

	y = FIX2INT(y);
	x = FIX2INT(x);

	for(i=0; i<num; i++)
	{
		//y
		s32 yy = y + pPat->cy;
		if(yy >= 160 || yy <= -16)
		{
			pPat++;
			continue;
		}

		//idx
		s32 arrayIdx = yy + 16;
		arrayIdx >>= 4;

		ST_SPR_DOUBLER* pV = &SprBuf[SprStatus.wIdx].sv[arrayIdx];
		if(pV->cnt >= MAX_SPR_RASTER_CNT)
		{
			return;
		}

		//x
		s32 xx = x + pPat->cx;
		if(xx > 240 || xx <= -16)
		{
			pPat++;
			continue;
		}

		pV->buf[pV->cnt].attr0 = (yy & 0x00ff) | ATTR0_COLOR_16 | ATTR0_NORMAL;
		pV->buf[pV->cnt].attr1 = (xx & 0x01ff) | pPat->flip | ATTR1_SIZE_16;
		pV->buf[pV->cnt].attr2 = PcgGetOamTile(pPat->idx) | ATTR2_PRIORITY(1) | pal;
		pV->cnt++;

		SprStatus.allCnt++;
		pPat++;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void SprSetVerti(s16 ofsY)
{
	SprStatus.ofsY = ofsY;
}
//---------------------------------------------------------------------------
IWRAM_CODE void SprSetHoriz(s16 ofsX)
{
	SprStatus.ofsX = ofsX;
}
//---------------------------------------------------------------------------
IWRAM_CODE void SpSetOffset(s16 ofsX, s16 ofsY)
{
	SprStatus.ofsX = ofsX;
	SprStatus.ofsY = ofsY;
}
//---------------------------------------------------------------------------
// XvCg]iVCOUNT 荞݁j
IWRAM_CODE void SprVcount()
{
	ST_SPR_STATUS*  pS = &SprStatus;
	ST_SPRITE32*    pO = (ST_SPRITE32*)OAM;
	ST_SPRITE32*    pM = (ST_SPRITE32*)&SprBuf[pS->dIdx].sv[pS->tCnt].buf[0];
	ST_SPR_DOUBLER* pD = (ST_SPR_DOUBLER*)&SprBuf[pS->dIdx].sv[pS->tCnt];

//	TRACEOUT("VCNT IN: %d %d\n", REG_VCOUNT, pS->tCnt);

	s16 allCnt = pS->sCnt + pD->cnt;

	s16 sad1 = 0;
	s16 dad1 = pS->sCnt;
	s16 cnt1 = (allCnt >= MAX_SPR_RASTER_CNT) ? (MAX_SPR_RASTER_CNT - 1) - pS->sCnt : pD->cnt;

	s16 sad2 = allCnt & (MAX_SPR_RASTER_CNT - 1);
	s16 dad2 = 0;
	s16 cnt2 = (allCnt >= MAX_SPR_RASTER_CNT) ? allCnt - MAX_SPR_RASTER_CNT : 0;

	if(cnt1 != 0)
	{
		REG_DMA2SAD = (u32)&pM[sad1];
		REG_DMA2DAD = (u32)&pO[dad1];
		REG_DMA2CNT = (u32)4*cnt1 | (DMA_SRC_INC | DMA_DST_INC | DMA16 | DMA_ENABLE);
	}

	if(cnt2 != 0)
	{
		REG_DMA2SAD = (u32)&pM[sad2];
		REG_DMA2DAD = (u32)&pO[dad2];
		REG_DMA2CNT = (u32)4*cnt2 | (DMA_SRC_INC | DMA_DST_INC | DMA16 | DMA_ENABLE);
	}

	pS->sCnt += pD->cnt;
	pS->sCnt &= (MAX_SPR_RASTER_CNT - 1);

#if 0
	for(i=0; i<pD->cnt; i++)
	{
		pO[ pS->sCnt ] = *pM++;

		pS->sCnt++;
		pS->sCnt &= (MAX_SPR_RASTER_CNT - 1);
	}

	for(i=0; i<MAX_SPR_RASTER_CNT - pD->cnt; i++)
	{
		pO[ pS->sCnt ] = unVisible;

		pS->sCnt++;
		pS->sCnt &= (MAX_SPR_RASTER_CNT - 1);
	}
#endif

#if 0
	//Ă̂Qi`܂j
	REG_DMA2SAD = (u32)pM;
	REG_DMA2DAD = (u32)OAM;
	REG_DMA2CNT = (u32)4*MAX_SPR_RASTER_CNT | (DMA_SRC_INC | DMA_DST_INC | DMA16 | DMA_ENABLE);
#endif

	pS->tCnt += 1;
	pS->vCnt += 16;
	if(pS->vCnt >= 228)
	{
		pS->vCnt -= 228;
	}

	REG_DISPSTAT = (REG_DISPSTAT & STAT_MASK) | LCDC_VCNT | VCOUNT(pS->vCnt);
//	TRACEOUT("VCNT OT: %d %d\n", REG_VCOUNT, pS->tCnt);
}
//---------------------------------------------------------------------------
IWRAM_CODE void SprVblank()
{
	ST_STATUS*      pM = &Status;
	ST_SPR_STATUS*  pS = &SprStatus;

	pM->frameCount++;
	pM->fpsTimer++;
	if(pM->fpsTimer == 60)
	{
		pM->fps = pM->fpsCount;
		pM->fpsTimer = 0;
		pM->fpsCount = 0;
	}

	pS->vCnt = 228 - 16 - PRE_RASTER_LINES;
	pS->tCnt = 0;
	pS->sCnt = 0;

	REG_DISPSTAT = (REG_DISPSTAT & STAT_MASK) | LCDC_VCNT | VCOUNT(pS->vCnt);
	REG_IRQ_WAITFLAGS |= IRQ_VBLANK;
}
