#ifndef __SPR_H__
#define __SPR_H__
#ifdef __cplusplus
extern "C" {
#endif


#include "lib/gba.h"

//---------------------------------------------------------------------------
#define MAX_SPR_BUF_CNT				1024
#define MAX_SPR_PRIORITY_CNT		64
#define MAX_SPR_RASTER_CNT			128
#define MAX_SPR_DIV_CNT				24

#define INT2FIX(A)					((A)<<7)
#define FIX2INT(A)					((A)>>7)

//X^ubN̓]^C~OύXB@̏ꍇ̓RgAEg
#define EMULATOR

#if defined(EMULATOR)
#	define PRE_RASTER_LINES			1
#else
#	define PRE_RASTER_LINES			2
#endif

#define SP_PAL(pal)					((pal) << 12)
#define SP_NOFLIP					((0x4000))
#define SP_HFLIP					((0x1000) | (0x4000))
#define SP_VFLIP					((0x2000) | (0x4000))
#define SP_HVFLIP					(SP_HFLIP | SP_VFLIP)

//---------------------------------------------------------------------------
typedef struct {
	u16 allCnt;
	s16 vCnt;
	s16 tCnt;
	s16 sCnt;
	u16 wIdx;
	u16 dIdx;
	s16 ofsX;
	s16 ofsY;
} ST_SPR_STATUS;

typedef struct {
	u16 attr0;
	u16 attr1;
	u16 attr2;
	u16 attr3;
} ST_SPRITE;

typedef struct {
	u32 attr01;
	u32 attr23;
} ST_SPRITE32;

typedef struct {
	s16 cnt;
	ST_SPRITE buf[MAX_SPR_RASTER_CNT];
} ST_SPR_DOUBLER;

typedef struct {
	ST_SPR_DOUBLER sv[12];
} ST_SPR_BUF;


//f[^\
typedef struct {
	u16 idx;	//
	s16 cx;		// center x
	s16 cy;		// center y
	u16 flip;	// default flip
} ST_SPR_PAT;

typedef struct {
	u16 regCnt;
	ST_SPR_PAT p[];
} ST_SPR_ANIME;

//---------------------------------------------------------------------------
EWRAM_CODE void SprInit();
EWRAM_CODE void SprSetAnime(void* pp);
IWRAM_CODE void SprClean();
IWRAM_CODE void SprSetBuf(u16 idx, s32 x, s32 y, u16 pal);

IWRAM_CODE void SprSetVerti(s16 ofsY);
IWRAM_CODE void SprSetHoriz(s16 ofsX);
IWRAM_CODE void SpSetOffset(s16 ofsX, s16 ofsY);

IWRAM_CODE void SprVcount();
IWRAM_CODE void SprVblank();




#ifdef __cplusplus
}
#endif
#endif
