#include "main.h"
#include "Key.h"
#include "mymem.h"
#include "spr_arm.h"
#include "res.h"
#include "lib/gamemath_arm.h"
#include "task.h"
#include "task_bullet.h"


//---------------------------------------------------------------------------
enum AnimeType {
	none,
	loop,
	dir,
};

static const struct {
	s32 ptn;
	s32 pal;
	enum AnimeType type;
	s32 ani_frames;
	s32 ani_wait;
} BulletPattern[] = {
	{ bullet1_a,  SP_PAL(14), loop, 4, 2, },
	{ bullet2_a,  SP_PAL(13), loop, 8, 1, },
	{ bullet3_a,  SP_PAL(13), loop, 4, 2, },
	{ bullet4_a,  SP_PAL(13), loop, 4, 2, },
	{ bullet5a_a, SP_PAL(14), dir,  4, 2, },
	{ bullet6_a,  SP_PAL(12), dir,  1, 0, },
	{ bullet7a_a, SP_PAL(12), dir,  2, 1, },
	{ bullet8a_a, SP_PAL(12), dir,  2, 1, },
	{ bullet9a_a, SP_PAL(12), dir,  2, 1, },
};


//---------------------------------------------------------------------------
EWRAM_CODE void TaskBulletAlloc(s32 x, s32 y, s32 deg, s32 speed, s32 chr)
{
	ST_TASK* pT = TaskGetFreePointer();

	if(pT == NULL)
	{
		return;
	}

	ST_TASK_BULLET* pM = (ST_TASK_BULLET*)MyMemAlloc(sizeof(ST_TASK_BULLET));

	pT->type       = TYPE_BULLET;
	pT->isRun      = TRUE;
	pT->isDraw     = TRUE;
	pT->pRun       = (void*)&TaskBulletRun;
	pT->pDraw      = (void*)&TaskBulletDraw;
	pT->pVal       = (void*)pM;

	pM->x          = x;
	pM->y          = y;
	pM->deg        = deg & 0xff;
	pM->speed      = speed;
	pM->chr        = chr;
	pM->animeWait  = 0;
	pM->animeFrame = 0;
	pM->pat        = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void TaskBulletRun(void* s)
{
	ST_TASK* t = (ST_TASK*)s;
	ST_TASK_BULLET* p = (ST_TASK_BULLET*)t->pVal;

	p->x += SIN(p->deg) * p->speed;
	p->y -= COS(p->deg) * p->speed;

	if(p->x > INT2FIX(240+16) || p->x < INT2FIX(-16) || p->y > INT2FIX(160+16) || p->y < INT2FIX(-16))
	{
		t->type   = TYPE_NOUSE;
		t->isRun  = FALSE;
		t->isDraw = FALSE;

		MyMemFree((u32)p, sizeof(ST_TASK_BULLET));
		return;
	}

	p->animeWait++;
	if(p->animeWait > BulletPattern[p->chr].ani_wait)
	{
		p->animeWait = 0;

		p->animeFrame++;
		if(p->animeFrame >= BulletPattern[p->chr].ani_frames)
		{
			p->animeFrame = 0;
		}
	}

	p->pat = BulletPattern[p->chr].ptn;
	switch(BulletPattern[p->chr].type)
	{
		case none:
			break;

		case loop:
			p->pat += p->animeFrame;
			break;

		case dir:
			p->pat += (((p->deg + 8)/16)&15) + p->animeFrame * 16;
	}

	SprSetBuf(p->pat, p->x, p->y, BulletPattern[p->chr].pal);
}
//---------------------------------------------------------------------------
EWRAM_CODE void TaskBulletDraw(void* s)
{
	ST_TASK* t = (ST_TASK*)s;
	ST_TASK_BULLET* p = (ST_TASK_BULLET*)t->pVal;
}
