#include "bg.h"
#include "main.h"
#include "key.h"
#include "mymem.h"
#include "std.h"
#include "spr_arm.h"
#include "task.h"
#include "task_myship.h"
#include "task_sys.h"

//---------------------------------------------------------------------------
//key.c
extern ST_KEY Key;
//main.c
extern ST_STATUS Status;
//spr_arm.c
extern ST_SPR_STATUS SprStatus;
extern ST_SPR_BUF    SprBuf[2];


//---------------------------------------------------------------------------
EWRAM_CODE void TaskSysAlloc()
{
	ST_TASK* pT = TaskGetFreePointer();

	if(pT == NULL)
	{
		return;
	}

	ST_TASK_SYS* pM = (ST_TASK_SYS*)MyMemAlloc(sizeof(ST_TASK_SYS));

	pT->type    = TYPE_SYSTEM;
	pT->isRun   = TRUE;
	pT->isDraw  = TRUE;
	pT->pRun    = (void*)&TaskSysRun;
	pT->pDraw   = (void*)&TaskSysDraw;
	pT->pVal    = (void*)pM;

	pM->curWait = 0;
	pM->cur     = 0;
	pM->isProc  = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void TaskSysRun(void* s)
{
	ST_TASK* t = (ST_TASK*)s;
	ST_TASK_SYS* p = (ST_TASK_SYS*)t->pVal;

	if(Key.trg & KEY_START)
	{
		TaskMaskRun(TYPE_MYSHIP, p->isProc);
		TaskMaskRun(TYPE_BULLET, p->isProc);

		p->isProc = (p->isProc == FALSE) ? TRUE : FALSE;
	}

	if(Key.trg & KEY_R && p->isProc == FALSE)
	{
		TaskMyShipAlloc();
	}

	if(p->isProc == FALSE)
	{
		return;
	}

	if(Key.trg & KEY_UP)
	{
		p->cur--;
		if(p->cur < 0) p->cur = 0;
	}

	if(Key.trg & KEY_DOWN)
	{
		p->cur++;
		if(p->cur > 2) p->cur = 2;
	}

	if(!(Key.rep & (KEY_LEFT | KEY_RIGHT)))
	{
		return;
	}
	s16 sign = (Key.rep & KEY_RIGHT) ? 1 : -1;

	switch(p->cur)
	{
	//ẽLN^
	case 0:
		Status.bulletChr += sign;
		if(Status.bulletChr < 0) Status.bulletChr = 0;
		if(Status.bulletChr > 8) Status.bulletChr = 8;
		break;

	//ẽXs[h
	case 1:
		Status.bulletSpeed += sign;
		if(Status.bulletSpeed < 1) Status.bulletSpeed = 1;
		break;

	//ẽEFCg
	case 2:
		Status.bulletWait += sign;
		if(Status.bulletWait < 0) Status.bulletWait = 0;
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void TaskSysDraw(void* s)
{
	ST_TASK* pSt = (ST_TASK*)s;
	ST_TASK_SYS* p = (ST_TASK_SYS*)pSt->pVal;
	char str[32];
	u16 i;

	_Sprintf(str, "DLY:%4d", Status.delay);
	BgPrint(0,0,16,str);
	_Sprintf(str, "FPS:%4d", Status.fps);
	BgPrint(0,0,17,str);
	_Sprintf(str, "BUL:%4d", TaskGetTypeCnt(TYPE_BULLET));
	BgPrint(0,0,18,str);
	_Sprintf(str, "SPR:%4d", SprStatus.allCnt);
	BgPrint(0,0,19,str);

	for(i=0; i<11; i++)
	{
		_Sprintf(str, "%3d", SprBuf[SprStatus.wIdx].sv[i].cnt);
		BgPrint(0,25,i,str);
	}

	if(p->isProc == TRUE)
	{
		Bg1On();
		_Sprintf(str, " CHAR :%3d", Status.bulletChr);
		BgPrint(1,0,0,str);
		_Sprintf(str, " SPEED:%3d", Status.bulletSpeed);
		BgPrint(1,0,1,str);
		_Sprintf(str, " WAIT :%3d", Status.bulletWait);
		BgPrint(1,0,2,str);
		BgPrint(1,0,p->cur,">");
	}
	else
	{
		Bg1Off();
	}
}
